<?php
/* Knockout Cup Plugin
 * by Martin Weber aka askuri
 * Version 0.1.0
 * Published on June 13th 2020
 * 
 * MIT License
 * 
 * Copyright (c) 2020 Martin Weber
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

Aseco::registerEvent('onSync',						'ko_onSync');
Aseco::registerEvent('onPlayerConnect',				'ko_onPlayerConnect');
Aseco::registerEvent('onPlayerManialinkPageAnswer', 'ko_onMlAnswer');
Aseco::registerEvent('onNewChallenge2',				'ko_onNewChallenge2');
Aseco::registerEvent('onEndRace1',					'ko_onEndRace1');
Aseco::registerEvent('onDonation',					'ko_onDonation');

Aseco::addChatCommand('ko', 'Knockout cup settings', false);

global $knockout;

class knockout {
	private $aseco;
	public $config = array();
	// private $next_status;
	public $last_phase_switch;
	public $players = array();
	public $status = 'warmup'; // remaining players
	
	public function __construct() {
		/*
		 * CONFIG
		 */
		$this->config['warmup'] 				= 14 * 24; // warmup phase before cup starts (in hours)
		$this->config['ko_time'] 				= 3.5 * 24; // time between knock-outs (in hours)
		$this->config['ko_begin_playercount'] 	= 70; // how many players are still in after warmup-end
		$this->config['ko_num_players'] 		= 5; // how many players are knocked out
		/*
		 * END CONFIG
		 */
		$this->loadData();
	}
	
	public function onSync($aseco) {
		$this->aseco = $aseco;
		// $this->showKOInfo($aseco);
	}
	
	public function onPlayerConnect($aseco, $player) {
		$this->showKOInfo($aseco, $player->login);
		// $this->showKOBar($aseco, $player->login);
		
		if ($this->status == 'warmup') {
			// add player to the list
			// $this->players[$player->login] = 0; // rank is 0
		}
		else {
			if (!isset($this->players[$player->login])) {
				$chat = '$d00>> Sorry, the cup has already started. You can\'t take part anymore :(';
				$aseco->client->query('ChatSendServerMessageToLogin', $chat, $player->login);
			}
			else {
				
				// TODO tell the player his current rank (and maybe avg)
				// tell him whether he is in danger to be kicked out
			}
		}
	}
	
	public function onMlAnswer($aseco, $answer) {
		switch ($answer[2]) {
			case 5451101: // top100 list
				// Close Records Eyepiece windows
				$aseco->releaseEvent('onPlayerManialinkPageAnswer', array($answer[0], $answer[1], 91800));
				
				// Close ulti windows
				$aseco->releaseEvent('onPlayerManialinkPageAnswer', array($answer[0], $answer[1], 5450102));
				
				$command = array (	'params' => '',
									'author' => $aseco->server->players->player_list[$answer[1]] // Get the player object
								 );
				chat_top100($aseco, $command);
				break;
		case 5451102: // welcome window
				// Close ulti windows
				$aseco->releaseEvent('onPlayerManialinkPageAnswer', array($answer[0], $answer[1], 5450102));
				
				$player = $aseco->server->players->player_list[$answer[1]]; // Get the player object
				re_buildWelcomeWindow(true, $player->login, $player->nickname);
				
				break;
		}
	}
	
	public function saveData() {
		$data = array();
		$data['status'] 			= $this->status;
		// $data['next_status'] 		= $this->next_status;
		$data['last_phase_switch']	= $this->last_phase_switch;
		file_put_contents('knockout.json', json_encode($data));
	}
	
	public function loadData() {
		$data = json_decode(file_get_contents('knockout.json'), true);
		
		if (!$data) {
			$this->status 				= 'warmup';
			// $this->next_status 			= $this->config['warmup'];
			$this->last_phase_switch 	= time() - 1; // -1 to prevent overlay of checks
		}
		else {
			$this->status 				= $data['status'];
			// $this->next_status 			= $data['next_status'];
			$this->last_phase_switch 	= $data['last_phase_switch'];
		}
		
	}
	
	// stolen and modified from plugin.rasp.php
	public function getAvg($login) {
		// global $minrank;

		$pid = $this->aseco->getPlayerId($login);
		$query = 'SELECT avg FROM rs_rank
		          WHERE playerID=' . $pid;
		$res = mysql_query($query);
		
		if (mysql_num_rows($res) > 0) { // player has an avg
			$row = mysql_fetch_array($res);
			
			// return round($row['avg'] / 10000, 1);
			return sprintf("%4.1F", $row['avg'] / 10000);
		}
		else { // avg can't be calculated yet
			return 'none';
		}
	}
	
	public function getAvgList($limit = 100) {
		if ($limit == 'warmup') {
			$limit_sql = '';
		}
		else {
			$limit_sql = ' LIMIT '.$limit;
		}
		
		$query = 'SELECT p.NickName,p.login, r.avg FROM players p
	          LEFT JOIN rs_rank r ON (p.Id=r.PlayerId)
	          WHERE r.avg!=0 ORDER BY r.avg ASC'.$limit_sql;
		$res = mysql_query($query);
		
		$i = 1;
		while ($row = mysql_fetch_assoc($res)) {
			$row['rank'] = $i;
			$array[$row['login']] = $row;
			$i++;
		}
		
		return $array;
	}
	
	// stolen and modified from plugin.rasp.php
	public function getRank($login) {
		$pid = $this->aseco->getPlayerId($login);
		$query = 'SELECT playerid FROM rs_rank ORDER BY avg ASC';
		$res = mysql_query($query);
		
		$rank = 1;
		while ($row = mysql_fetch_array($res)) {
			if ($row['playerid'] == $pid) break;
			$rank++;
		}
		
		return $rank;
	}
	
	public function showKOInfo($aseco, $login = null) {
		if ($login == null) {
			foreach ($aseco->server->players->player_list as $player) {
				$this->showKOInfo($aseco, $player->login);
			}
		}
		else {
			$background = 'style="Bgs1InRace" substyle="NavButton"';
			$rank = $this->getRank($login);
			$avg = $this->getAvg($login);
			
			if ($this->status == 'warmup') {
				$player_status = '';
				$cup_status = 'Cup status: $ff3Warmup$fff-phase';
				$cup_status2 = 'Warmup ends in $FF3'.round($this->timeTillPhaseSwitch()/3600, 1) .'$fff hour(s).'; // 3600
			}
			elseif ($this->status >= $this->config['ko_num_players']) { // cup running
				if (isset($this->players[$login])) {
					$player_status = '$dfdYou\'re still in the cup! Good luck!';
				}
				else {
					$player_status = 'You\'re $d00knocked out$fff or unranked!';
				}
				$cup_status = 'Cup is $ff3running$fff. $FF3'.$this->status.'$fff players are still in.';
				$cup_status2 = 'Next knock-out in $FF3'.round($this->timeTillPhaseSwitch()/3600, 1).'$fff hour(s)!'; // 3600
			}
			else { // cup ended
				$background = 'style="Icons128x128_1" substyle="ShareBlink"';
				
				$player_status = '$F3CThe cup has ended! Thanks for taking part!';
				if (reset($this->players)['login'] == 'onkelh') {
					$cup_status = '$f00Cheaters don\'t deserve coppers!';
					$cup_status2 = '$F3CJust kidding. Congrats Henning :)';
				}
				else {
					$cup_status = '$f3c'.reset($this->players)['NickName'].'$f3c is the winner!!';
					$cup_status2 = '$F3CCongratulations! :)';
				}
			}
			
			// get coppers
			$aseco->client->query('GetServerCoppers');
			$coppers = $aseco->client->getResponse();
			
			$xml = '<manialink id="5452101">
			<quad posn="-65 45.5 -1" sizen="25 12" '.$background.' action="5451102"/>
			<label posn="-63.5 45.0 0" sizen="23 4" text="Your current cup rank: $FF3'.$rank.'$z (Avg: $FF3'.$avg.'$z)" textsize="1" textscale="1.1" />
			<label posn="-63.5 43.2 0" sizen="23 4" text="'.$player_status.'" textsize="1" textscale="1.1" />
			<label posn="-63.5 41.4 0" sizen="23 4" text="'.$cup_status.'" textsize="1" textscale="1.1" />
			<label posn="-63.5 39.6 0" sizen="23 4" text="'.$cup_status2.'" textsize="1" textscale="1.1" />
			<label posn="-63.5 37.8 0" sizen="23 4" text="There are currently $ff3'.$coppers.'$fffcc in the pot!" textsize="1" textscale="1.1" />
			<label posn="-63.5 36 0" sizen="23 4" text="Click here to see the $ff3rules$fff again." textsize="1" textscale="1.1" />
			';
			
			$xml.= '</manialink>';
			
			$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($login, $xml, 0, false));
			
			// ko bar
			if ($this->status == 'warmup') {
				$divider = $this->config['ko_begin_playercount'] / $rank;
			}
			else {
				$divider = $this->status / $rank;
			}
			
			if ($divider < 1) {
				$divider = 1;
			}
			
			if ($divider >= 1 && $avg != 'none') {
				$xml = '<manialink id="5452102"><frame posn="0 48.1 0">
					<quad posn="0 0 1" sizen="20 2" style="BgRaceScore2" substyle="CupFinisher" halign="center" />
					<quad posn="0 0 2" sizen="20 2" style="BgRaceScore2" substyle="CupFinisher" halign="center" />
					<quad posn="0 0 0" sizen="20 2" bgcolor="F00F" halign="center" />
					<quad posn="0 0 5" sizen="20 2" image="http://maniacdn.net/askuri/misc/spectrumredyellowgreen.jpg" halign="center" />
					<quad posn="0 2 7" sizen="21 5" style="Bgs1InRace" substyle="BgCard3" halign="center" />
					<quad posn="'. (9-(18 / $divider)) .' 0 6" sizen="0.3 2" bgcolor="000F" />
					<label posn="-9 -1.7 10" sizen="30 1" textsize="1" scale="0.65" textcolor="000F" text="KNOCKOUT-METER"/>
				</frame></manialink>';
				
			}
			else {
				$xml = '<manialink id="5452102">/manialink>';
			}
			
			$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($login, $xml, 0, false));
		}
	}
	
	public function hideKOInfo($aseco) {
		$xml = '<manialink id="5452101"></manialink>'; // info widget
		$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));	
		
		$xml = '<manialink id="5452102"></manialink>'; // ko bar
		$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));	
	}
	
	/*
	public function showKOBar($aseco, $login = null) {
		if ($login == null) {
			foreach ($aseco->server->players->player_list as $player) {
				$this->showKOBar($aseco, $player->login);
			}
		}
		else {
			$xml = '<manialink id="5452102"><frame posn="0 48.1 0">
				<quad posn="0 0 1" sizen="20 2" style="BgRaceScore2" substyle="CupFinisher" halign="center" />
				<quad posn="0 0 2" sizen="20 2" style="BgRaceScore2" substyle="CupFinisher" halign="center" />
				<quad posn="0 0 0" sizen="20 2" bgcolor="F00F" halign="center" />
				<quad posn="0 0 5" sizen="20 2" image="http://maniacdn.net/askuri/misc/spectrumredyellowgreen.jpg" halign="center" />
				<quad posn="0 2 7" sizen="21 5" style="Bgs1InRace" substyle="BgCard3" halign="center" />
			</frame></manialink>';
			$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($login, $xml, 0, false));
		}
	}
	
	public function hideKOBar($aseco) {
		$xml = '<manialink id="5452102"></manialink>';
		$aseco->client->addCall('SendDisplayManialinkPage', array($xml, 0, false));	
	}
	*/
	
	public function checkNextStatus($aseco) {
		if ($this->timeTillPhaseSwitch() <= 0) { // check needed
			$old_status = $this->status;
			if ($old_status == 'warmup') {
				$this->status = $this->config['ko_begin_playercount']; // warmup ended
				$chat = '>> $0f0The warmup ended and the first players have just been knocked out.';
				$aseco->client->query('ChatSendServerMessage', $chat);
			}
			elseif ($old_status <= $this->config['ko_num_players']) {
				$this->status = 1; // skip if cup has ended
				$chat = '>> $0f0The cup is over! Thanks for playing! You can see the winner in the top-left corner.';
				$aseco->client->query('ChatSendServerMessage', $chat);
				return;
			}
			else {
				$chat = '>> $0f0Knockout! $ff0The next players are out now.';
				$aseco->client->query('ChatSendServerMessage', $chat);
				$this->status = $old_status - $this->config['ko_num_players']; // kick players out
			}

			$this->last_phase_switch = time(); // reset countdown
		}
	}
	
	public function timeTillPhaseSwitch() {
		return $this->last_phase_switch + $this->config['ko_time'] * 3600 - time(); // 3600
	}
}

function ko_onSync($aseco) {
	global $knockout;
	$knockout->onSync($aseco);
}
function ko_onPlayerConnect($aseco, $player) {
	global $knockout;
	$knockout->onPlayerConnect($aseco, $player);
}
function ko_onMlAnswer($aseco, $answer) {
	global $knockout;
	$knockout->onMlAnswer($aseco, $answer);
}
function ko_onNewChallenge2($aseco, $challenge_item) {
	global $knockout;
	$knockout->players = $knockout->getAvgList($knockout->status);
	$knockout->showKOInfo($aseco);
	// $knockout->showKOBar($aseco);
	$knockout->saveData();
}
function ko_onEndRace1($aseco, $race) {
	global $knockout;
	$knockout->hideKOInfo($aseco);
	// $knockout->hideKOBar($aseco);
	$knockout->checkNextStatus($aseco);
}
function ko_onDonation($aseco, $donation) {
	global $knockout;
	$knockout->showKOInfo($aseco); // refresh
}
function chat_ko($aseco, $command) {
	global $knockout;
	
	$player = $command['author'];
	$action = explode(' ', $command['params']);
	
	if ($aseco->isMasterAdminL($player->login)) {
		switch ($action[0]) {
			case 'reset':
				$knockout->status = 'warmup';
				$knockout->last_phase_switch = time() + $knockout->config['warmup'] * 3600 - $knockout->config['ko_time'] * 3600;
				ko_onNewChallenge2($aseco, null);
				break;
			case 'skip':
				if ($knockout->status == 'warmup') {
					$knockout->status = $knockout->config['ko_begin_playercount'];
				}
				else {
					$knockout->status -= $knockout->config['ko_num_players'];
				}
				
				$knockout->last_phase_switch = time();
				ko_onNewChallenge2($aseco, null);
			case 'refresh':
				$knockout->showKOInfo($aseco);
				break;
			case 'setphasetime':
				$knockout->config['ko_time'] = $action[1];
				break;
			case 'addphasetime':
				$knockout->last_phase_switch += $action[1] * 3600;
				break;
			default:
				$chat = array('reset (restart cup and go to warmup, locals are kept)',
							'skip (skip current phase)',
							'refresh (refresh info widget)',
							'setphasetime (changes the time between knockouts in hours)',
							'addphasetime (makes current phase last x hours longer)'
							);
				foreach ($chat as $msg) {
					$aseco->client->query('ChatSendServerMessageToLogin', $msg, $player->login);
				}
		}
		$knockout->showKOInfo($aseco);
	}
}

$knockout = new knockout();

?>